<?php
namespace Noalyss\Forms;
require_once NOALYSS_INCLUDE."/lib/ac_common.php";
/*
 *   This file is part of NOALYSS.
 *
 *   NOALYSS is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   NOALYSS is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with NOALYSS; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
// Copyright Author Dany De Bontridder danydb@aevalys.eu 31/10/23
/*! 
 * \file
 * \brief record the input from the web page
 */

/*!
 * \class
 * \brief record the input from the web page
 */

class Data_Record
{
    /**
     * @brief action to do before committing, could be implemented. If parameters are needed must use
     * @return bool true if succedd , false otherwise
     */
    protected function precommit():bool
    {
        return true;
    }

    /**
     * @brief action to do after committing , could be implemented
     * @return bool true if succedd , false otherwise
     */
    protected function postcommit():bool
    {
        return true;
    }

    /**
     * @brief action to take if precommit returns false
     * @return void
     */
    protected function error_precommit() {

    }

    /**
     * @brief action to take if postcommit returns false
     * @return void
     */
    protected function error_postcommit() {

    }
    /**
     * @brief find the corresponding form and record it, the form action can be a post or get
     * @code

    // Example of page receiving the data

    require_once '/home/dany/developpement/phpcompta/noalyss/include/constant.php';
    require_once '/home/dany/developpement/phpcompta/noalyss/include/ext/noalyss-form/nform_constant_min.php';

    \Noalyss\Forms\Data_Record::record();
     *
     * @endcode
     * @param $p_method = POST , GET or REQUEST , way to take data
     * @param $resend = 'all' make an URL with the parameter and add them to the url of the answer
     */
    public function record($p_method='REQUEST',$resend='all')
    {
        $http=new \HttpInput();

        try {
            if ( ! in_array($p_method,['REQUEST','POST','GET'] )) {
                throw new \Exception('DR55'."method [$p_method] invalide: possible options are POST, GET or REQUEST");
            }
            $cn=\Dossier::connect();
            if ($p_method == "GET")            $token=$http->get("token");
            if ($p_method == "POST")            $token=$http->post("token");
            if ($p_method == "REQUEST")            $token=$http->request("token");
            $dr_from=$_SERVER['REMOTE_ADDR']??'';
            if ($cn->get_value('select count(*) from noalyss_form.form where 
                                           f_token=$1 and (f_url_allow is null or f_url_allow=$2)'
                    ,[$token,$dr_from])==0) {
                throw new \Exception ("Unknown request token $token from $dr_from");
            }
        } catch (\Exception $e) {
            \record_log($e->getMessage());
            throw $e;
        }

        try {
            $form_id=$cn->get_value("select f_id from noalyss_form.form where f_token=$1",[$token]);
            $cn->start();
            $data_received_raw=new Data_received_raw_SQL($cn);
            $data_received_raw->setp('dr_from',$dr_from);
            $data_received_raw->insert();
            $form_detail_sql=new Form_detail_SQL($cn);
            $a_form_detail_sql=$form_detail_sql->collect_objects ('where form_id=$1',[$form_id]);
            foreach ($a_form_detail_sql as $form_detail_sql) {
                $parm=$form_detail_sql->getp('fd_post_param');

                if ($p_method == "GET")            $value=$http->get($parm);
                if ($p_method == "POST")            $value=$http->post($parm);
                if ($p_method == "REQUEST")            $value=$http->request($parm);


                $data_received_raw_sql=new Data_received_raw_detail_SQL($cn);
                $data_received_raw_sql->setp('form_detail_id',$form_detail_sql->getp('fd_id'));
                $data_received_raw_sql->setp('drd_post_value',$value);
                $data_received_raw_sql->setp('data_received_raw_id',$data_received_raw->dr_id);
                $data_received_raw_sql->insert();
            }
            if ( $this->precommit() == false ) {
                $this->error_precommit();
            }
            $cn->commit();
            if ( $this->postcommit() == false ) {
                $this->error_postcommit();
            }

            $url_redirect =$cn->get_value("select f_url_success from noalyss_form.form where f_id=$1",[$form_id]);
            if ( ! empty ($url_redirect)) {
                $url=($resend == 'all')?http_build_query($_REQUEST):'';
                $url_redirect=Noalyss_Form::url_add_param($url_redirect,$url);
                Noalyss_Form::redirect($url_redirect);
            }
        } catch (\Exception $e) {
            $cn->rollback();
            echo $e->getMessage();
            $url_redirect =$cn->get_value("select f_url_success from noalyss_form.form where f_id=$1",[$form_id]);
            if ( ! empty ($url_redirect)) {
                $url=($resend == 'all')?http_build_query($_REQUEST):'';
                $url_redirect=Noalyss_Form::url_add_param($url_redirect,$url);
                Noalyss_Form::redirect($url_redirect);
            }

        }
    }
}